from __future__ import annotations

from typing import DefaultDict
from typing import Mapping
from typing import Type

from openapi_spec_validator.validation.keywords import KeywordValidator


class KeywordValidatorRegistry(DefaultDict[str, KeywordValidator]):
    def __init__(
        self, keyword_validators: Mapping[str, Type[KeywordValidator]]
    ):
        super().__init__()
        self.keyword_validators = keyword_validators

    def __missing__(self, keyword: str) -> KeywordValidator:
        if keyword not in self.keyword_validators:
            raise KeyError(keyword)
        cls = self.keyword_validators[keyword]
        self[keyword] = cls(self)
        return self[keyword]



"""OpenAPI spec validator validation proxies module."""
import warnings
from typing import Any
from typing import Hashable
from typing import Iterator
from typing import Mapping
from typing import Optional
from typing import Tuple

from jsonschema.exceptions import ValidationError
from jsonschema_path.typing import Schema

from openapi_spec_validator.validation.exceptions import OpenAPIValidationError
from openapi_spec_validator.validation.exceptions import ValidatorDetectError
from openapi_spec_validator.validation.types import SpecValidatorType


class SpecValidatorProxy:
    def __init__(
        self,
        cls: SpecValidatorType,
        deprecated: str = "SpecValidator",
        use: Optional[str] = None,
    ):
        self.cls = cls

        self.deprecated = deprecated
        self.use = use or self.cls.__name__

    def validate(
        self,
        schema: Schema,
        base_uri: str = "",
        spec_url: Optional[str] = None,
    ) -> None:
        for err in self.iter_errors(
            schema,
            base_uri=base_uri,
            spec_url=spec_url,
        ):
            raise err

    def is_valid(self, schema: Schema) -> bool:
        error = next(self.iter_errors(schema), None)
        return error is None

    def iter_errors(
        self,
        schema: Schema,
        base_uri: str = "",
        spec_url: Optional[str] = None,
    ) -> Iterator[ValidationError]:
        warnings.warn(
            f"{self.deprecated} is deprecated. Use {self.use} instead.",
            DeprecationWarning,
        )
        validator = self.cls(schema, base_uri=base_uri, spec_url=spec_url)
        return validator.iter_errors()


class DetectValidatorProxy:
    def __init__(self, choices: Mapping[Tuple[str, str], SpecValidatorProxy]):
        self.choices = choices

    def detect(self, instance: Mapping[Hashable, Any]) -> SpecValidatorProxy:
        for (key, value), validator in self.choices.items():
            if key in instance and instance[key].startswith(value):
                return validator
        raise ValidatorDetectError("Spec schema version not detected")

    def validate(
        self,
        instance: Mapping[Hashable, Any],
        base_uri: str = "",
        spec_url: Optional[str] = None,
    ) -> None:
        validator = self.detect(instance)
        for err in validator.iter_errors(
            instance, base_uri=base_uri, spec_url=spec_url
        ):
            raise err

    def is_valid(self, instance: Mapping[Hashable, Any]) -> bool:
        validator = self.detect(instance)
        error = next(validator.iter_errors(instance), None)
        return error is None

    def iter_errors(
        self,
        instance: Mapping[Hashable, Any],
        base_uri: str = "",
        spec_url: Optional[str] = None,
    ) -> Iterator[OpenAPIValidationError]:
        warnings.warn(
            "openapi_spec_validator_proxy is deprecated.",
            DeprecationWarning,
        )
        validator = self.detect(instance)
        yield from validator.iter_errors(
            instance, base_uri=base_uri, spec_url=spec_url
        )



from openapi_spec_validator.validation.proxies import DetectValidatorProxy
from openapi_spec_validator.validation.proxies import SpecValidatorProxy
from openapi_spec_validator.validation.validators import OpenAPIV2SpecValidator
from openapi_spec_validator.validation.validators import (
    OpenAPIV30SpecValidator,
)
from openapi_spec_validator.validation.validators import (
    OpenAPIV31SpecValidator,
)

__all__ = [
    "openapi_v2_spec_validator",
    "openapi_v3_spec_validator",
    "openapi_v30_spec_validator",
    "openapi_v31_spec_validator",
    "openapi_spec_validator_proxy",
    "OpenAPIV2SpecValidator",
    "OpenAPIV3SpecValidator",
    "OpenAPIV30SpecValidator",
    "OpenAPIV31SpecValidator",
]

# v2.0 spec
openapi_v2_spec_validator = SpecValidatorProxy(
    OpenAPIV2SpecValidator,
    deprecated="openapi_v2_spec_validator",
    use="OpenAPIV2SpecValidator",
)

# v3.0 spec
openapi_v30_spec_validator = SpecValidatorProxy(
    OpenAPIV30SpecValidator,
    deprecated="openapi_v30_spec_validator",
    use="OpenAPIV30SpecValidator",
)

# v3.1 spec
openapi_v31_spec_validator = SpecValidatorProxy(
    OpenAPIV31SpecValidator,
    deprecated="openapi_v31_spec_validator",
    use="OpenAPIV31SpecValidator",
)

# alias to the latest v3 version
openapi_v3_spec_validator = openapi_v31_spec_validator
OpenAPIV3SpecValidator = OpenAPIV31SpecValidator

# detect version spec
openapi_spec_validator_proxy = DetectValidatorProxy(
    {
        ("swagger", "2.0"): openapi_v2_spec_validator,
        ("openapi", "3.0"): openapi_v30_spec_validator,
        ("openapi", "3.1"): openapi_v31_spec_validator,
    },
)



"""OpenAIP spec validator schemas utils module."""
import sys
from os import path
from typing import Any
from typing import Hashable
from typing import Mapping
from typing import Tuple

if sys.version_info >= (3, 9):
    from importlib.resources import as_file
    from importlib.resources import files
else:
    from importlib_resources import as_file
    from importlib_resources import files

from jsonschema_path.readers import FilePathReader


def get_schema(version: str) -> Tuple[Mapping[Hashable, Any], str]:
    schema_path = f"resources/schemas/v{version}/schema.json"
    ref = files("openapi_spec_validator") / schema_path
    with as_file(ref) as resource_path:
        schema_path_full = path.join(path.dirname(__file__), resource_path)
    return FilePathReader(schema_path_full).read()


def get_schema_content(version: str) -> Mapping[Hashable, Any]:
    content, _ = get_schema(version)
    return content
